@@ -2,7 +2,8 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2015-2016 Cirilo Bernardo <cirilo.bernardo@gmail.com>
- * Copyright (C) 2018-2020 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2018-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2022 CERN
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,7 +30,6 @@
 
 #include <wx/datetime.h>
 #include <wx/dir.h>
-#include <wx/filename.h>
 #include <wx/log.h>
 #include <wx/stdpaths.h>
 
@@ -55,12 +55,12 @@
 #include <project.h>
 #include <settings/common_settings.h>
 #include <settings/settings_manager.h>
+#include <wx_filename.h>
 
 
 #define MASK_3D_CACHE "3D_CACHE"
 
 static std::mutex mutex3D_cache;
-static std::mutex mutex3D_cacheManager;
 
 
 static bool isSHA1Same( const unsigned char* shaA, const unsigned char* shaB ) noexcept
@@ -196,28 +196,18 @@ S3D_CACHE::~S3D_CACHE()
 {
     FlushCache();
 
-    // We'll delete ".3dc" cache files older than this many days
-    int clearCacheInterval = 0;
-
-    if( Pgm().GetCommonSettings() )
-        clearCacheInterval = Pgm().GetCommonSettings()->m_System.clear_3d_cache_interval;
-
-    // An interval of zero means the user doesn't want to ever clear the cache
-
-    if( clearCacheInterval > 0 )
-        CleanCacheDir( clearCacheInterval );
-
     delete m_FNResolver;
     delete m_Plugins;
 }
 
 
-SCENEGRAPH* S3D_CACHE::load( const wxString& aModelFile, S3D_CACHE_ENTRY** aCachePtr )
+SCENEGRAPH* S3D_CACHE::load( const wxString& aModelFile, const wxString& aBasePath,
+                             S3D_CACHE_ENTRY** aCachePtr )
 {
     if( aCachePtr )
         *aCachePtr = nullptr;
 
-    wxString full3Dpath = m_FNResolver->ResolvePath( aModelFile );
+    wxString full3Dpath = m_FNResolver->ResolvePath( aModelFile, aBasePath );
 
     if( full3Dpath.empty() )
     {
@@ -282,9 +272,9 @@ SCENEGRAPH* S3D_CACHE::load( const wxString& aModelFile, S3D_CACHE_ENTRY** aCach
 }
 
 
-SCENEGRAPH* S3D_CACHE::Load( const wxString& aModelFile )
+SCENEGRAPH* S3D_CACHE::Load( const wxString& aModelFile, const wxString& aBasePath )
 {
-    return load( aModelFile );
+    return load( aModelFile, aBasePath );
 }
 
 
@@ -305,8 +295,7 @@ SCENEGRAPH* S3D_CACHE::checkCache( const wxString& aFileName, S3D_CACHE_ENTRY**
         // or we do not have a configured cache file directory, we create an
         // entry to prevent further attempts at loading the file
 
-        if( m_CacheMap.insert( std::pair< wxString, S3D_CACHE_ENTRY* >
-            ( aFileName, ep ) ).second == false )
+        if( m_CacheMap.emplace( aFileName, ep ).second == false )
         {
             wxLogTrace( MASK_3D_CACHE,
                         wxT( "%s:%s:%d\n * [BUG] duplicate entry in map file; key = '%s'" ),
@@ -324,8 +313,7 @@ SCENEGRAPH* S3D_CACHE::checkCache( const wxString& aFileName, S3D_CACHE_ENTRY**
         return nullptr;
     }
 
-    if( m_CacheMap.insert( std::pair< wxString, S3D_CACHE_ENTRY* >
-                               ( aFileName, ep ) ).second == false )
+    if( m_CacheMap.emplace( aFileName, ep ).second == false )
     {
         wxLogTrace( MASK_3D_CACHE,
                     wxT( "%s:%s:%d\n * [BUG] duplicate entry in map file; key = '%s'" ),
@@ -393,7 +381,13 @@ bool S3D_CACHE::getSHA1( const wxString& aFileName, unsigned char* aSHA1Sum )
 
     fclose( fp );
     unsigned int digest[5];
-    dblock.get_digest( digest );
+    // V8 only
+    // Boost 1.86 and later changed digest_type from uchar[20] from int[5]
+    // But KiCad 8.99 and later use MurmurHash3 here, so just do a fairly ugly cast to keep
+    // this going for another few months.
+    static_assert( sizeof( digest ) == sizeof( boost::uuids::detail::sha1::digest_type& ),
+                   "SHA1 digest size mismatch" );
+    dblock.get_digest( reinterpret_cast<boost::uuids::detail::sha1::digest_type&>( digest ) );
 
     // ensure MSB order
     for( int i = 0; i < 5; ++i )
@@ -514,7 +508,7 @@ bool S3D_CACHE::Set3DConfigDir( const wxString& aConfigDir )
 
     wxFileName cfgdir( ExpandEnvVarSubstitutions( aConfigDir, m_project ), wxEmptyString );
 
-    cfgdir.Normalize();
+    cfgdir.Normalize( FN_NORMALIZE_FLAGS );
 
     if( !cfgdir.DirExists() )
     {
@@ -643,10 +637,10 @@ void S3D_CACHE::ClosePlugins()
 }
 
 
-S3DMODEL* S3D_CACHE::GetModel( const wxString& aModelFileName )
+S3DMODEL* S3D_CACHE::GetModel( const wxString& aModelFileName, const wxString& aBasePath )
 {
     S3D_CACHE_ENTRY* cp = nullptr;
-    SCENEGRAPH* sp = load( aModelFileName, &cp );
+    SCENEGRAPH*      sp = load( aModelFileName, aBasePath,&cp );
 
     if( !sp )
         return nullptr;
@@ -708,40 +702,4 @@ void S3D_CACHE::CleanCacheDir( int aNumDaysOld )
             }
         }
     }
-}
-
-
-S3D_CACHE* PROJECT::Get3DCacheManager( bool aUpdateProjDir )
-{
-    std::lock_guard<std::mutex> lock( mutex3D_cacheManager );
-
-    // Get the existing cache from the project
-    S3D_CACHE* cache = dynamic_cast<S3D_CACHE*>( GetElem( ELEM_3DCACHE ) );
-
-    if( !cache )
-    {
-        // Create a cache if there is not one already
-        cache = new S3D_CACHE();
-
-        wxFileName cfgpath;
-        cfgpath.AssignDir( SETTINGS_MANAGER::GetUserSettingsPath() );
-        cfgpath.AppendDir( wxT( "3d" ) );
-
-        cache->SetProgramBase( &Pgm() );
-        cache->Set3DConfigDir( cfgpath.GetFullPath() );
-
-        SetElem( ELEM_3DCACHE, cache );
-        aUpdateProjDir = true;
-    }
-
-    if( aUpdateProjDir )
-        cache->SetProject( this );
-
-    return cache;
-}
-
-
-FILENAME_RESOLVER* PROJECT::Get3DFilenameResolver()
-{
-    return Get3DCacheManager()->GetResolver();
-}
+}
\ No newline at end of file
