@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2017 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 2017-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2017-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -145,49 +145,54 @@ double PLOTTER::GetDashGapLenIU( int aLineWidth ) const
     return userToDeviceSize( m_renderSettings->GetGapLength( aLineWidth ) );
 }
 
-
-void PLOTTER::Arc( const VECTOR2I& aCenter, const VECTOR2I& aStart, const VECTOR2I& aEnd,
-                   FILL_T aFill, int aWidth, int aMaxError )
+#include <wx/log.h>
+void PLOTTER::Arc( const VECTOR2D& aStart, const VECTOR2D& aMid, const VECTOR2D& aEnd, FILL_T aFill,
+                   int aWidth )
 {
+    VECTOR2D aCenter = CalcArcCenter( aStart, aMid, aEnd );
+
     EDA_ANGLE startAngle( aStart - aCenter );
     EDA_ANGLE endAngle( aEnd - aCenter );
-    int       radius = ( aStart - aCenter ).EuclideanNorm();
 
-    if( startAngle > endAngle )
-    {
-        if( endAngle < ANGLE_0 )
-            endAngle.Normalize();
-        else
-            startAngle = startAngle.Normalize() - ANGLE_360;
-    }
+    // < 0: left, 0 : on the line, > 0 : right
+    double det = ( aEnd - aStart ).Cross( aMid - aStart );
 
-    // In old Kicad code, calls to Arc() using angles calls this function after
-    // swapping angles and negate them (to compensate the inverted Y axis).
-    // So to be compatible with Arc() calls with angles, do the same thing
-    std::swap( startAngle, endAngle );
-    startAngle = -startAngle;
-    endAngle = -endAngle;
+    int       cw = det <= 0;
+    EDA_ANGLE angle = endAngle - startAngle;
 
-    Arc( aCenter, startAngle, endAngle, radius, aFill, aWidth );
+    if( cw )
+        angle.Normalize();
+    else
+        angle.NormalizeNegative();
+
+    double radius = ( aStart - aCenter ).EuclideanNorm();
+    Arc( aCenter, startAngle, angle, radius, aFill, aWidth );
 }
 
 
-void PLOTTER::Arc( const VECTOR2I& aCenter, const EDA_ANGLE& aStartAngle,
-                   const EDA_ANGLE& aEndAngle, int aRadius, FILL_T aFill, int aWidth )
+void PLOTTER::Arc( const VECTOR2D& aCenter, const EDA_ANGLE& aStartAngle, const EDA_ANGLE& aAngle,
+                   double aRadius, FILL_T aFill, int aWidth )
 {
-    EDA_ANGLE       startAngle( aStartAngle );
-    EDA_ANGLE       endAngle( aEndAngle );
+    polyArc( aCenter, aStartAngle, aAngle, aRadius, aFill, aWidth );
+}
+
+
+void PLOTTER::polyArc( const VECTOR2D& aCenter, const EDA_ANGLE& aStartAngle,
+                       const EDA_ANGLE& aAngle, double aRadius, FILL_T aFill, int aWidth )
+{
+    EDA_ANGLE       startAngle = aStartAngle;
+    EDA_ANGLE       endAngle = startAngle + aAngle;
     const EDA_ANGLE delta( 5.0, DEGREES_T ); // increment to draw arc
     VECTOR2I        start, end;
-    const int       sign = -1;
+    const int       sign = 1;
 
-    if( startAngle > endAngle )
+    if( aAngle < ANGLE_0 )
         std::swap( startAngle, endAngle );
 
     SetCurrentLineWidth( aWidth );
 
-    start.x = aCenter.x + KiROUND( aRadius * startAngle.Cos() );
-    start.y = aCenter.y + sign*KiROUND( aRadius * startAngle.Sin() );
+    start.x = KiROUND( aCenter.x + aRadius * startAngle.Cos() );
+    start.y = KiROUND( aCenter.y + sign * aRadius * startAngle.Sin() );
 
     if( aFill != FILL_T::NO_FILL )
     {
@@ -201,13 +206,13 @@ void PLOTTER::Arc( const VECTOR2I& aCenter, const EDA_ANGLE& aStartAngle,
 
     for( EDA_ANGLE ii = startAngle + delta; ii < endAngle; ii += delta )
     {
-        end.x = aCenter.x + KiROUND( aRadius * ii.Cos() );
-        end.y = aCenter.y + sign*KiROUND( aRadius * ii.Sin() );
+        end.x = KiROUND( aCenter.x + aRadius * ii.Cos() );
+        end.y = KiROUND( aCenter.y + sign * aRadius * ii.Sin() );
         LineTo( end );
     }
 
-    end.x = aCenter.x + KiROUND( aRadius * endAngle.Cos() );
-    end.y = aCenter.y + sign*KiROUND( aRadius * endAngle.Sin() );
+    end.x = KiROUND( aCenter.x + aRadius * endAngle.Cos() );
+    end.y = KiROUND( aCenter.y + sign * aRadius * endAngle.Sin() );
 
     if( aFill != FILL_T::NO_FILL )
     {
@@ -533,16 +538,16 @@ void PLOTTER::sketchOval( const VECTOR2I& aPos, const VECTOR2I& aSize, const EDA
         corners[ii] += aPos;
     }
 
-    // Gen shape:
+    // Gen shape (2 lines and 2 180 deg arcs):
     MoveTo( corners[0] );
     FinishTo( corners[1] );
 
-    Arc( corners[2], orient + ANGLE_180, orient + ANGLE_360, radius, FILL_T::NO_FILL );
+    Arc( corners[2], -orient, ANGLE_180, radius, FILL_T::NO_FILL );
 
     MoveTo( corners[3] );
     FinishTo( corners[4] );
 
-    Arc( corners[5], orient, orient + ANGLE_180, radius, FILL_T::NO_FILL );
+    Arc( corners[5], -orient, -ANGLE_180, radius, FILL_T::NO_FILL );
 }
 
 
@@ -570,58 +575,47 @@ void PLOTTER::ThickSegment( const VECTOR2I& start, const VECTOR2I& end, int widt
 }
 
 
-void PLOTTER::ThickArc( const VECTOR2I& centre, const EDA_ANGLE& aStartAngle,
-                        const EDA_ANGLE& aEndAngle, int aRadius, int aWidth,
+void PLOTTER::ThickArc( const VECTOR2D& centre, const EDA_ANGLE& aStartAngle,
+                        const EDA_ANGLE& aAngle, double aRadius, int aWidth,
                         OUTLINE_MODE aTraceMode, void* aData )
 {
     if( aTraceMode == FILLED )
     {
-        Arc( centre, aStartAngle, aEndAngle, aRadius, FILL_T::NO_FILL, aWidth );
+        Arc( centre, aStartAngle, aAngle, aRadius, FILL_T::NO_FILL, aWidth );
     }
     else
     {
         SetCurrentLineWidth( -1 );
-        Arc( centre, aStartAngle, aEndAngle, aRadius - ( aWidth - m_currentPenWidth ) / 2,
+        Arc( centre, aStartAngle, aAngle, aRadius - ( aWidth - m_currentPenWidth ) / 2,
              FILL_T::NO_FILL, -1 );
-        Arc( centre, aStartAngle, aEndAngle, aRadius + ( aWidth - m_currentPenWidth ) / 2,
+        Arc( centre, aStartAngle, aAngle, aRadius + ( aWidth - m_currentPenWidth ) / 2,
              FILL_T::NO_FILL, -1 );
     }
 }
 
 
-void PLOTTER::ThickArc( const VECTOR2I& aCentre, const VECTOR2I& aStart,
-                        const VECTOR2I& aEnd, int aWidth,
-                        OUTLINE_MODE aTraceMode, void* aData )
+void PLOTTER::ThickArc( const EDA_SHAPE& aArcShape, OUTLINE_MODE aTraceMode, void* aData )
 {
-    if( aTraceMode == FILLED )
-    {
-        Arc( aCentre, aStart, aEnd, FILL_T::NO_FILL, aWidth, GetPlotterArcHighDef() );
-    }
+    VECTOR2D center = aArcShape.getCenter();
+    VECTOR2D mid = aArcShape.GetArcMid();
+    VECTOR2D start = aArcShape.GetStart();
+    VECTOR2D end = aArcShape.GetEnd();
+
+    EDA_ANGLE startAngle( start - center );
+    EDA_ANGLE endAngle( end - center );
+    EDA_ANGLE angle = endAngle - startAngle;
+
+    // < 0: left, 0 : on the line, > 0 : right
+    double det = ( end - start ).Cross( mid - start );
+
+    if( det <= 0 ) // cw
+        angle.Normalize();
     else
-    {
-        SetCurrentLineWidth( -1 );
-        int radius = ( aStart - aCentre ).EuclideanNorm();
+        angle.NormalizeNegative();
 
-        int new_radius = radius - ( aWidth - m_currentPenWidth ) / 2;
-        VECTOR2I start = ( aStart - aCentre ).Resize( new_radius ) + aCentre;
-        VECTOR2I end = ( aEnd - aCentre ).Resize( new_radius ) + aCentre;
+    double radius = ( start - center ).EuclideanNorm();
 
-        Arc( aCentre, start, end, FILL_T::NO_FILL, -1, GetPlotterArcHighDef() );
-
-        new_radius = radius + ( aWidth - m_currentPenWidth ) / 2;
-        start = ( aStart - aCentre ).Resize( new_radius ) + aCentre;
-        end = ( aEnd - aCentre ).Resize( new_radius ) + aCentre;
-
-        Arc( aCentre, start, end, FILL_T::NO_FILL, -1, GetPlotterArcHighDef() );
-    }
-}
-
-
-void PLOTTER::ThickArc( const EDA_SHAPE& aArcShape,
-                           OUTLINE_MODE aTraceMode, void* aData )
-{
-    ThickArc( aArcShape.getCenter(),aArcShape.GetStart(), aArcShape.GetEnd(),
-              aArcShape.GetWidth(), aTraceMode, aData );
+    ThickArc( center, startAngle, angle, radius, aArcShape.GetWidth(), aTraceMode, aData );
 }
 
 
@@ -694,41 +688,24 @@ void PLOTTER::PlotPoly( const SHAPE_LINE_CHAIN& aCornerList, FILL_T aFill, int a
 }
 
 
-/**
- * Same as GRText, but plot graphic text instead of draw it.
- *
- * @param aPos is the text position (according to aH_justify, aV_justify).
- * @param aColor is the text color.
- * @param aText is the text to draw.
- * @param aOrient is the angle.
- * @param aSize is the text size (size.x or size.y can be < 0 for mirrored texts).
- * @param aH_justify is the horizontal justification (Left, center, right).
- * @param aV_justify is the vertical justification (bottom, center, top).
- * @param aPenWidth is the line width (if = 0, use plot default line width).
- * @param aItalic is the true to simulate an italic font.
- * @param aBold use true to use a bold font Useful only with default width value (aPenWidth = 0).
- * @param aMultilineAllowed use true to plot text as multiline, otherwise single line.
- * @param aData is a parameter used by some plotters in SetCurrentLineWidth(),
- *              not directly used here.
- */
-void PLOTTER::Text( const VECTOR2I&             aPos,
-                    const COLOR4D&              aColor,
-                    const wxString&             aText,
-                    const EDA_ANGLE&            aOrient,
-                    const VECTOR2I&             aSize,
-                    enum GR_TEXT_H_ALIGN_T      aH_justify,
-                    enum GR_TEXT_V_ALIGN_T      aV_justify,
-                    int                         aPenWidth,
-                    bool                        aItalic,
-                    bool                        aBold,
-                    bool                        aMultilineAllowed,
-                    KIFONT::FONT*               aFont,
-                    void*                       aData )
+void PLOTTER::Text( const VECTOR2I&       aPos,
+                    const COLOR4D&         aColor,
+                    const wxString&        aText,
+                    const EDA_ANGLE&       aOrient,
+                    const VECTOR2I&        aSize,
+                    enum GR_TEXT_H_ALIGN_T aH_justify,
+                    enum GR_TEXT_V_ALIGN_T aV_justify,
+                    int                    aPenWidth,
+                    bool                   aItalic,
+                    bool                   aBold,
+                    bool                   aMultilineAllowed,
+                    KIFONT::FONT*          aFont,
+                    const KIFONT::METRICS& aFontMetrics,
+                    void*                  aData )
 {
     KIGFX::GAL_DISPLAY_OPTIONS empty_opts;
 
     SetColor( aColor );
-    SetCurrentLineWidth( aPenWidth, aData );
 
     if( aPenWidth == 0 && aBold ) // Use default values if aPenWidth == 0
         aPenWidth = GetPenSizeForBold( std::min( aSize.x, aSize.y ) );
@@ -740,8 +717,9 @@ void PLOTTER::Text( const VECTOR2I&             aPos,
             // Stroke callback
             [&]( const VECTOR2I& aPt1, const VECTOR2I& aPt2 )
             {
-                MoveTo( (wxPoint) aPt1 );
-                LineTo( (wxPoint) aPt2 );
+                SetCurrentLineWidth( aPenWidth );
+                MoveTo( aPt1 );
+                LineTo( aPt2 );
                 PenFinish();
             },
             // Polygon callback
@@ -759,8 +737,59 @@ void PLOTTER::Text( const VECTOR2I&             aPos,
     attributes.m_Valign = aV_justify;
     attributes.m_Size = aSize;
 
+    // if Size.x is < 0, the text is mirrored (we have no other param to know a text is mirrored)
+    if( attributes.m_Size.x < 0 )
+    {
+        attributes.m_Size.x = -attributes.m_Size.x;
+        attributes.m_Mirrored = true;
+    }
+
     if( !aFont )
         aFont = KIFONT::FONT::GetFont();
 
-    aFont->Draw( &callback_gal, aText, aPos, attributes );
+    aFont->Draw( &callback_gal, aText, aPos, attributes, aFontMetrics );
+}
+
+void PLOTTER::PlotText( const VECTOR2I&        aPos,
+                        const COLOR4D&         aColor,
+                        const wxString&        aText,
+                        const TEXT_ATTRIBUTES& aAttributes,
+                        KIFONT::FONT*          aFont,
+                        const KIFONT::METRICS& aFontMetrics,
+                        void*                  aData )
+{
+    KIGFX::GAL_DISPLAY_OPTIONS empty_opts;
+
+    TEXT_ATTRIBUTES attributes = aAttributes;
+    int penWidth = attributes.m_StrokeWidth;
+
+    SetColor( aColor );
+    SetCurrentLineWidth( penWidth, aData );
+
+    if( penWidth == 0 && attributes.m_Bold ) // Use default values if aPenWidth == 0
+        penWidth = GetPenSizeForBold( std::min( attributes.m_Size.x, attributes.m_Size.y ) );
+
+    if( penWidth < 0 )
+        penWidth = -penWidth;
+
+    attributes.m_StrokeWidth = penWidth;
+
+    CALLBACK_GAL callback_gal( empty_opts,
+            // Stroke callback
+            [&]( const VECTOR2I& aPt1, const VECTOR2I& aPt2 )
+            {
+                MoveTo( aPt1 );
+                LineTo( aPt2 );
+                PenFinish();
+            },
+            // Polygon callback
+            [&]( const SHAPE_LINE_CHAIN& aPoly )
+            {
+                PlotPoly( aPoly, FILL_T::FILLED_SHAPE, 0, aData );
+            } );
+
+    if( !aFont )
+        aFont = KIFONT::FONT::GetFont();
+
+    aFont->Draw( &callback_gal, aText, aPos, attributes, aFontMetrics );
 }
