@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2013 CERN
- * Copyright (C) 2013-2021 KiCad Developers, see CHANGELOG.txt for contributors.
+ * Copyright (C) 2013-2024 KiCad Developers, see CHANGELOG.txt for contributors.
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
  * This program is free software; you can redistribute it and/or
@@ -23,24 +23,19 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
-#include <ignore.h>
+#include <core/ignore.h>
 #include <macros.h>
 #include <trace_helpers.h>
-
 #include <tool/tool_manager.h>
-#include <tool/tools_holder.h>
 #include <tool/tool_dispatcher.h>
 #include <tool/actions.h>
 #include <tool/action_manager.h>
 #include <tool/action_menu.h>
 #include <view/view.h>
 #include <view/wx_view_controls.h>
-
-#include <class_draw_panel_gal.h>
 #include <eda_draw_frame.h>
-
-#include <core/arraydim.h>
-#include <core/optional.h>
+#include <core/kicad_algo.h>
+#include <optional>
 #include <wx/log.h>
 #include <wx/stc/stc.h>
 #include <wx/settings.h>
@@ -113,18 +108,23 @@ struct TOOL_DISPATCHER::BUTTON_STATE
         case BUT_RIGHT:
             return mouseState.RightIsDown();
 
-        default:
-            assert( false );
-            break;
-        }
+        case BUT_AUX1:
+            return mouseState.Aux1IsDown();
 
-        return false;
+        case BUT_AUX2:
+            return mouseState.Aux2IsDown();
+
+        default:
+            wxFAIL_MSG( wxT( "unknown button" ) );
+            return false;
+        }
     }
 };
 
 
 TOOL_DISPATCHER::TOOL_DISPATCHER( TOOL_MANAGER* aToolMgr ) :
-    m_toolMgr( aToolMgr )
+        m_toolMgr( aToolMgr ),
+        m_currentMenu( nullptr )
 {
     m_sysDragMinX = wxSystemSettings::GetMetric( wxSYS_DRAG_X );
     m_sysDragMinY = wxSystemSettings::GetMetric( wxSYS_DRAG_Y );
@@ -138,6 +138,10 @@ TOOL_DISPATCHER::TOOL_DISPATCHER( TOOL_MANAGER* aToolMgr ) :
                          wxEVT_RIGHT_UP, wxEVT_RIGHT_DCLICK ) );
     m_buttons.push_back( new BUTTON_STATE( BUT_MIDDLE, wxEVT_MIDDLE_DOWN,
                          wxEVT_MIDDLE_UP, wxEVT_MIDDLE_DCLICK ) );
+    m_buttons.push_back( new BUTTON_STATE( BUT_AUX1, wxEVT_AUX1_DOWN,
+                         wxEVT_AUX1_UP, wxEVT_AUX1_DCLICK ) );
+    m_buttons.push_back( new BUTTON_STATE( BUT_AUX2, wxEVT_AUX2_DOWN,
+                         wxEVT_AUX2_UP, wxEVT_AUX2_DCLICK ) );
 
     ResetState();
 }
@@ -167,7 +171,7 @@ bool TOOL_DISPATCHER::handleMouseButton( wxEvent& aEvent, int aIndex, bool aMoti
 {
     BUTTON_STATE* st = m_buttons[aIndex];
     wxEventType type = aEvent.GetEventType();
-    OPT<TOOL_EVENT> evt;
+    std::optional<TOOL_EVENT> evt;
     bool isClick = false;
 
 //    bool up = type == st->upEvent;
@@ -269,21 +273,13 @@ bool isKeySpecialCode( int aKeyCode )
 {
     // These keys have predefined actions (like move thumbtrack cursor),
     // and we do not want these actions executed
-    const enum wxKeyCode special_keys[] =
+    const std::vector<enum wxKeyCode> special_keys =
     {
         WXK_PAGEUP, WXK_PAGEDOWN,
         WXK_NUMPAD_PAGEUP, WXK_NUMPAD_PAGEDOWN
     };
 
-    bool isInList = false;
-
-    for( unsigned ii = 0; ii < arrayDim( special_keys ) && !isInList; ii++ )
-    {
-        if( special_keys[ii] == aKeyCode )
-            isInList = true;
-    }
-
-    return isInList;
+    return alg::contains( special_keys, aKeyCode );
 }
 
 
@@ -292,29 +288,22 @@ bool isKeySpecialCode( int aKeyCode )
 // that is not used alone in kicad
 static bool isKeyModifierOnly( int aKeyCode )
 {
-    const enum wxKeyCode special_keys[] =
+    static std::vector<enum wxKeyCode> special_keys =
     {
-        WXK_CONTROL, WXK_RAW_CONTROL, WXK_SHIFT,WXK_ALT
+        WXK_CONTROL, WXK_RAW_CONTROL, WXK_SHIFT, WXK_ALT
     };
 
-    bool isInList = false;
-
-    for( unsigned ii = 0; ii < arrayDim( special_keys ) && !isInList; ii++ )
-    {
-        if( special_keys[ii] == aKeyCode )
-            isInList = true;
-    }
-
-    return isInList;
+    return alg::contains( special_keys, aKeyCode );
 }
 
 
 static bool isMouseClick( wxEventType type )
 {
-    return type == wxEVT_LEFT_DOWN || type == wxEVT_LEFT_UP || type == wxEVT_MIDDLE_DOWN
-           || type == wxEVT_MIDDLE_UP || type == wxEVT_RIGHT_DOWN || type == wxEVT_RIGHT_UP
-           || type == wxEVT_LEFT_DCLICK || type == wxEVT_MIDDLE_DCLICK
-           || type == wxEVT_RIGHT_DCLICK;
+    return type == wxEVT_LEFT_DOWN || type == wxEVT_LEFT_UP || type == wxEVT_LEFT_DCLICK
+           || type == wxEVT_MIDDLE_DOWN || type == wxEVT_MIDDLE_UP || type == wxEVT_MIDDLE_DCLICK
+           || type == wxEVT_RIGHT_DOWN || type == wxEVT_RIGHT_UP || type == wxEVT_RIGHT_DCLICK
+           || type == wxEVT_AUX1_DOWN || type == wxEVT_AUX1_UP || type == wxEVT_AUX1_DCLICK
+           || type == wxEVT_AUX2_DOWN || type == wxEVT_AUX2_UP || type == wxEVT_AUX2_DCLICK;
 }
 
 
@@ -347,9 +336,9 @@ int translateSpecialCode( int aKeyCode )
 }
 
 
-OPT<TOOL_EVENT> TOOL_DISPATCHER::GetToolEvent( wxKeyEvent* aKeyEvent, bool* keyIsSpecial )
+std::optional<TOOL_EVENT> TOOL_DISPATCHER::GetToolEvent( wxKeyEvent* aKeyEvent, bool* keyIsSpecial )
 {
-    OPT<TOOL_EVENT> evt;
+    std::optional<TOOL_EVENT> evt;
     int             key = aKeyEvent->GetKeyCode();
     int             unicode_key = aKeyEvent->GetUnicodeKey();
 
@@ -360,7 +349,7 @@ OPT<TOOL_EVENT> TOOL_DISPATCHER::GetToolEvent( wxKeyEvent* aKeyEvent, bool* keyI
         return evt;
     }
 
-    wxLogTrace( kicadTraceKeyEvent, "TOOL_DISPATCHER::GetToolEvent %s", dump( *aKeyEvent ) );
+    wxLogTrace( kicadTraceKeyEvent, wxS( "TOOL_DISPATCHER::GetToolEvent %s" ), dump( *aKeyEvent ) );
 
     // if the key event must be skipped, skip it here if the event is a wxEVT_CHAR_HOOK
     // and do nothing.
@@ -419,7 +408,7 @@ OPT<TOOL_EVENT> TOOL_DISPATCHER::GetToolEvent( wxKeyEvent* aKeyEvent, bool* keyI
 #endif
 
     if( key == WXK_ESCAPE ) // ESC is the special key for canceling tools
-        evt = TOOL_EVENT( TC_COMMAND, TA_CANCEL_TOOL );
+        evt = TOOL_EVENT( TC_COMMAND, TA_CANCEL_TOOL, WXK_ESCAPE );
     else
         evt = TOOL_EVENT( TC_KEYBOARD, TA_KEY_PRESSED, key | mods );
 
@@ -432,7 +421,7 @@ void TOOL_DISPATCHER::DispatchWxEvent( wxEvent& aEvent )
     bool            motion = false;
     bool            buttonEvents = false;
     VECTOR2D        pos;
-    OPT<TOOL_EVENT> evt;
+    std::optional<TOOL_EVENT> evt;
     bool            keyIsEscape  = false;  // True if the keypress was the escape key
     bool            keyIsSpecial = false;  // True if the key is a special key code
     wxWindow*       focus = wxWindow::FindFocus();
@@ -452,7 +441,7 @@ void TOOL_DISPATCHER::DispatchWxEvent( wxEvent& aEvent )
     {
         wxWindow* holderWindow = dynamic_cast<wxWindow*>( m_toolMgr->GetToolHolder() );
 
-#if defined( _WIN32 )
+#if defined( _WIN32 ) || defined( __WXGTK__ )
         // Mouse events may trigger regardless of window status (windows feature)
         // However we need to avoid focus fighting (especially modals)
         if( holderWindow && KIPLATFORM::UI::IsWindowActive( holderWindow ) )
@@ -466,16 +455,17 @@ void TOOL_DISPATCHER::DispatchWxEvent( wxEvent& aEvent )
 
     if( isMouseClick( type ) )
     {
-        if( m_toolMgr->GetToolHolder() && m_toolMgr->GetToolHolder()->GetToolCanvas() )
+        if( m_toolMgr->GetToolHolder() && m_toolMgr->GetToolHolder()->GetToolCanvas() &&
+            !m_toolMgr->GetToolHolder()->GetToolCanvas()->HasFocus() )
+        {
             m_toolMgr->GetToolHolder()->GetToolCanvas()->SetFocus();
+        }
     }
 
     // Mouse handling
     // Note: wxEVT_LEFT_DOWN event must always be skipped.
     if( type == wxEVT_MOTION || type == wxEVT_MOUSEWHEEL ||
-#if wxCHECK_VERSION( 3, 1, 0 ) || defined( USE_OSX_MAGNIFY_EVENT )
         type == wxEVT_MAGNIFY ||
-#endif
         isMouseClick( type ) ||
         // Event issued when mouse retains position in screen coordinates,
         // but changes in world coordinates (e.g. autopanning)
@@ -512,7 +502,7 @@ void TOOL_DISPATCHER::DispatchWxEvent( wxEvent& aEvent )
     {
         wxKeyEvent* ke = static_cast<wxKeyEvent*>( &aEvent );
 
-        wxLogTrace( kicadTraceKeyEvent, "TOOL_DISPATCHER::DispatchWxEvent %s", dump( *ke ) );
+        wxLogTrace( kicadTraceKeyEvent, wxS( "TOOL_DISPATCHER::DispatchWxEvent %s" ), dump( *ke ) );
 
         // Do not process wxEVT_CHAR_HOOK for a shift-modified key, as ACTION_MANAGER::RunHotKey
         // will run the un-shifted key and that's not what we want.  Wait to get the translated
@@ -548,6 +538,23 @@ void TOOL_DISPATCHER::DispatchWxEvent( wxEvent& aEvent )
     }
     else if( type == wxEVT_MENU_OPEN || type == wxEVT_MENU_CLOSE || type == wxEVT_MENU_HIGHLIGHT )
     {
+        wxMenuEvent* tmp = dynamic_cast<wxMenuEvent*>( &aEvent );
+
+        // Something is amiss if the event has these types and isn't a menu event, so bail out on
+        // its processing
+        if( !tmp )
+        {
+            aEvent.Skip();
+            return;
+        }
+
+        wxMenuEvent& menuEvent = *tmp;
+
+#if wxCHECK_VERSION( 3, 2, 0 )
+        // Forward the event to the menu for processing
+        if( ACTION_MENU* currentMenu = dynamic_cast<ACTION_MENU*>( menuEvent.GetMenu() ) )
+            currentMenu->OnMenuEvent( menuEvent );
+#else
         //
         // wxWidgets has several issues that we have to work around:
         //
@@ -562,29 +569,26 @@ void TOOL_DISPATCHER::DispatchWxEvent( wxEvent& aEvent )
         //    hotkey.  So we keep track of menu highlighting so we can differentiate.
         //
 
-        static ACTION_MENU* currentMenu;
-
-        wxMenuEvent& menuEvent = *dynamic_cast<wxMenuEvent*>( &aEvent );
-
         if( type == wxEVT_MENU_OPEN )
         {
-            currentMenu = dynamic_cast<ACTION_MENU*>( menuEvent.GetMenu() );
+            m_currentMenu = dynamic_cast<ACTION_MENU*>( menuEvent.GetMenu() );
 
-            if( currentMenu )
-                currentMenu->OnMenuEvent( menuEvent );
+            if( m_currentMenu )
+                m_currentMenu->OnMenuEvent( menuEvent );
         }
         else if( type == wxEVT_MENU_HIGHLIGHT )
         {
-            if( currentMenu )
-                currentMenu->OnMenuEvent( menuEvent );
+            if( m_currentMenu )
+                m_currentMenu->OnMenuEvent( menuEvent );
         }
         else if( type == wxEVT_MENU_CLOSE )
         {
-            if( currentMenu )
-                currentMenu->OnMenuEvent( menuEvent );
+            if( m_currentMenu )
+                m_currentMenu->OnMenuEvent( menuEvent );
 
-            currentMenu = nullptr;
+            m_currentMenu = nullptr;
         }
+#endif
 
         aEvent.Skip();
     }
@@ -593,12 +597,12 @@ void TOOL_DISPATCHER::DispatchWxEvent( wxEvent& aEvent )
 
     if( evt )
     {
-        wxLogTrace( kicadTraceToolStack, "TOOL_DISPATCHER::DispatchWxEvent %s", evt->Format() );
+        wxLogTrace( kicadTraceToolStack, wxS( "TOOL_DISPATCHER::DispatchWxEvent %s" ), evt->Format() );
 
         handled = m_toolMgr->ProcessEvent( *evt );
 
-        wxLogTrace( kicadTraceToolStack, "TOOL_DISPATCHER::DispatchWxEvent - Handled: %s  %s",
-                    ( handled ? "true" : "false" ), evt->Format() );
+        wxLogTrace( kicadTraceToolStack, wxS( "TOOL_DISPATCHER::DispatchWxEvent - Handled: %s  %s" ),
+                    ( handled ? wxS( "true" ) : wxS( "false" ) ), evt->Format() );
     }
 
     // pass the event to the GUI, it might still be interested in it
@@ -623,11 +627,13 @@ void TOOL_DISPATCHER::DispatchWxEvent( wxEvent& aEvent )
     // Escape key presses are never skipped by the handler since they correspond to tool cancel
     // events, and if they aren't skipped then they are propagated to other frames (which we
     // don't want).
-    if( (type == wxEVT_CHAR || type == wxEVT_CHAR_HOOK)
-         && !keyIsSpecial
-         && !handled
-         && !keyIsEscape )
+    if( ( type == wxEVT_CHAR || type == wxEVT_CHAR_HOOK )
+             && !keyIsSpecial
+             && !handled
+             && !keyIsEscape )
+    {
         aEvent.Skip();
+    }
 
     wxLogTrace( kicadTraceToolStack, "TOOL_DISPATCHER::DispatchWxEvent - Wx event skipped: %s",
                 ( aEvent.GetSkipped() ? "true" : "false" ) );
