@@ -19,7 +19,6 @@
 
 #include <algorithm>
 
-#include "base/cxx17_backports.h"
 #include "base/logging.h"
 #include "client/ios_handler/in_process_intermediate_dump_handler.h"
 #include "client/prune_crash_reports.h"
@@ -93,7 +92,7 @@ bool InProcessHandler::Initialize(
     upload_thread_options.identify_client_via_url = true;
 
     upload_thread_.reset(new CrashReportUploadThread(
-        database_.get(), url, upload_thread_options, callback));
+        database_.get(), url, "", upload_thread_options, callback));
   }
 
   if (!CreateDirectory(database))
